package eu.ztsh.wymiana.web.controller;

import eu.ztsh.wymiana.exception.InsufficientFundsException;
import eu.ztsh.wymiana.exception.UserNotFoundException;
import eu.ztsh.wymiana.service.CurrencyService;
import eu.ztsh.wymiana.web.model.CurrencyExchangeRequest;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequiredArgsConstructor
@Validated
@RestController
@RequestMapping(path = "/api/exchange", produces = "application/json")
public class ExchangeController {

    private final CurrencyService currencyService;

    @PostMapping
    public ResponseEntity<Object> exchange(@Valid @RequestBody CurrencyExchangeRequest request) {
        try {
            return ResponseEntity.status(200).body(currencyService.exchange(request));
        } catch (Exception e) {
            var status = switch (e) {
                case InsufficientFundsException ignored -> HttpStatus.BAD_REQUEST;
                case UserNotFoundException ignored -> HttpStatus.NOT_FOUND;
                default -> HttpStatus.INTERNAL_SERVER_ERROR;
            };
            return ResponseEntity.status(status).body(e.getMessage());
        }
    }

}
