package eu.ztsh.wymiana.web.controller;

import eu.ztsh.wymiana.exception.UserAlreadyExistsException;
import eu.ztsh.wymiana.exception.UserNotFoundException;
import eu.ztsh.wymiana.model.User;
import eu.ztsh.wymiana.service.UserService;
import eu.ztsh.wymiana.validation.ValidationFailedException;
import eu.ztsh.wymiana.web.model.UserCreateRequest;
import jakarta.validation.Valid;
import jakarta.validation.ValidationException;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequiredArgsConstructor
@Validated
@RestController
@RequestMapping(path = "/api/user", produces = "application/json")
public class UserController {

    private final UserService userService;

    @GetMapping("{pesel}")
    public ResponseEntity<User> get(@PathVariable("pesel") String pesel) {
        try {
            return userService.get(pesel)
                .map(ResponseEntity::ok)
                .orElseThrow(() -> new UserNotFoundException(pesel));
        } catch (Exception e) {
            return ResponseEntity.status(switch (e) {
                case ValidationException ignore -> 400;
                case UserNotFoundException ignore -> 404;
                default -> 500;
            }).build();
        }
    }

    @PostMapping
    public ResponseEntity<String> create(@Valid @RequestBody UserCreateRequest request) {
        try {
            userService.create(request);
        } catch (Exception e) {
            var status = switch (e) {
                case ValidationFailedException ignored -> HttpStatus.BAD_REQUEST;
                case UserAlreadyExistsException ignored -> HttpStatus.CONFLICT;
                default -> HttpStatus.INTERNAL_SERVER_ERROR;
            };
            return ResponseEntity.status(status).body(e.getMessage());
        }
        return ResponseEntity.status(204).build();
    }

}
